/*
 *
 * nucleo/core/select/sTimeKeeper.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/core/select/sTimeKeeper.H>
#include <nucleo/utils/TimeUtils.H>

#include <sys/time.h>

#include <iostream>

namespace nucleo {

  // -----------------------------------------------------------------------------

  std::list<sTimeKeeper*> sTimeKeeper::_instances ;

  sTimeKeeper::sTimeKeeper(void) {
    _instances.push_back(this) ;
  }

  sTimeKeeper::~sTimeKeeper() {
    _instances.remove(this) ;
  }

  void
  sTimeKeeper::_checkTime(void) {
    if (! (_state&ARMED)) return ; 
    long timeleft = _epoch - getCurrentTimeAsMillisecs() ;
    if (timeleft <= 0) {
	 _state = _repeat ? TRIGGERED_AND_ARMED : TRIGGERED ;
	 notifyObservers() ;
	 _epoch = getCurrentTimeAsMillisecs() + _delay ;
    }
  }

  void
  sTimeKeeper::arm(unsigned long milliseconds, bool repeat) {
    _repeat = repeat ;
    _delay = milliseconds ;
    _epoch = getCurrentTimeAsMillisecs() + milliseconds ;
    _state = ARMED ;
  }

  void
  sTimeKeeper::disarm(void) {
    _state = DISARMED ;
  }

  long
  sTimeKeeper::getTimeLeft(void) {
    if (_state==DISARMED) return -1 ;
    if (_state==TRIGGERED) return 0 ;
    long timeleft = _epoch - getCurrentTimeAsMillisecs() ;
    return (timeleft<0) ? 0 : timeleft ;
  }

  // -----------------------------------------------------------------------------

}
