
#ifndef n_imagefileImageSource_H
#define n_imagefileImageSource_H

/*
 *
 * nucleo/image/source/imagefileImageSource.H --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/utils/FileUtils.H>
#include <nucleo/image/source/ImageSource.H>
#include <nucleo/core/URI.H>
#include <nucleo/image/encoding/Conversion.H>

#include <stdexcept>

namespace nucleo {

  class imagefileImageSource : public ImageSource {

  private:

    std::string _filename ;
    Image::Encoding _source_encoding, _target_encoding ;
    Image _image ;
    bool _active ;

  public:

    imagefileImageSource(Image::Encoding target_encoding, const URI &uri,
					Image::Encoding source_encoding) {
	 _filename = uri.isOpaque() ? uri.getOpaque() : uri.getPath() ;
	 _target_encoding = target_encoding ;
	 _source_encoding = source_encoding ;
	 _active = false ;
    }

    bool start(void) {
	 if (_active) return false ;

	 unsigned int size = getFileSize(_filename.c_str()) ;
	 unsigned char *data = Image::AllocMem(size) ;
	 readFromFile(_filename.c_str(), data, size) ;

	 _image.setEncoding(_source_encoding) ;
	 _image.setData(data, size, Image::FREEMEM) ;
	 convertImage(&_image, _target_encoding) ;

	 _active = true ;
	 _chrono.start() ;
	 _chrono.tick() ;
	 notifyObservers() ;

	 return true ;
    }

    bool isActive(void) {
	 return _active ;
    }

    bool getNextImage(Image *img) {
	 if (!_active) return false ;
	 img->linkDataFrom(_image) ;
#if 0
	 std::cerr << _pendingNotifications << " notifications pending for " << this 
			 << " (" << _observers.size() << " observers)"
			 << std::endl ;
#endif
	 if (_pendingNotifications<1) stop() ;
	 return true ;
    }

    bool stop(void) {
	 if (!_active) return false ;
	 _chrono.stop() ;
	 _active = false ;
	 return true ;
    }

  } ;

  // ------------------------------------------------------------------

}

#endif
