/*
 *
 * nucleo/image/source/nudpImageSource.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/image/source/nudpImageSource.H>

#include <stdexcept>

namespace nucleo {

  // ------------------------------------------------------------------

  nudpImageSource::nudpImageSource(Image::Encoding target_encoding,
							const URI &uri) {
    _hostOrGroup = uri.getHost() ; 
    _port = uri.getPort() ;

    std::string query = uri.getQuery() ;

    std::string encoding ;
    _source_encoding = Image::JPEG ;    
    if (URI::getQueryArg(query, "encoding", &encoding))
	 _source_encoding = Image::getEncodingByName(encoding) ;
    
    _target_encoding = target_encoding==Image::PREFERRED ? _source_encoding : target_encoding ;

    _state = CLOSED ;
    _udp = 0 ;
    _data = 0 ;
    _dataSize = 0 ;
  }

  bool
  nudpImageSource::start(void) {
    if (_state!=CLOSED) return false ;

    try {
	 if (_hostOrGroup=="" || _hostOrGroup=="localhost" || _hostOrGroup=="127.0.0.1")
	   _udp = new UdpReceiver(_port) ;
	 else
	   _udp = new UdpReceiver(_hostOrGroup.c_str(), _port) ;
	 
	 int rcvbuf ;
	 for (unsigned int i=30; i>0; --i) {
	   rcvbuf = 1<<i ;
	   if (_udp->setBufferSize(rcvbuf)) break ;
	 }
	 std::cerr << "nudpImageSource: SO_RCVBUF=" << rcvbuf << std::endl ;

	 subscribeTo(_udp) ;

	 _state = OPEN ;
	 _chrono.start() ;
	 return true ;
    } catch (std::runtime_error e) {
	 std::cerr << "nudpImageSource: " << e.what() << std::endl ;
    } catch(...) {
    }

    delete _udp ;
    return false ;
  }

  void
  nudpImageSource::react(Observable *obs) {
    if (obs==_udp && _state!=CLOSED) {
	 if (_state==READY) {
	   delete [] _data ;
	   _state = OPEN ;
	 }
	 if (_udp->receive(&_data, &_dataSize)) {
	   _state = READY ;
	   if (!_pendingNotifications) notifyObservers() ;
	 }
    }
  }

  bool
  nudpImageSource::getNextImage(Image *img) {
    if (_state!=READY) return false ; 

    img->setEncoding(_source_encoding) ;
    img->setData(_data, _dataSize, Image::DELETE) ;
    _state = OPEN ;
    if (!convertImage(img, _target_encoding)) return false ;
    _chrono.tick() ;
    return true ;
  }

  bool
  nudpImageSource::stop(void) {
    if (_state==CLOSED) return false ;

    _chrono.stop() ;
    unsubscribeFrom(_udp) ;
    delete _udp ;
    _state = CLOSED ;
    _data = 0 ;

    return true ;
  }

  // ------------------------------------------------------------------

}
