/*
 *
 * nucleo/network/rdv/ServiceAnnouncer.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/network/rdv/ServiceAnnouncer.H>

#ifdef __APPLE__
#include <nucleo/network/rdv/mDNSServiceAnnouncer.H>
#else
#include <nucleo/network/rdv/dummyServiceAnnouncer.H>
#endif

#include <nucleo/core/PluginManager.H>

#include <iostream>
#include <stdexcept>

namespace nucleo {

  ServiceAnnouncer *
  ServiceAnnouncer::create(const char *name, const char *type, int port,
					  TextRecords *textRecords,
					  bool autorename,
					  const char *domain, const char *host) {
#ifdef __APPLE__
    return new mDNSServiceAnnouncer(name,type,port,textRecords,autorename,domain,host) ;
#else
    try {
	 ServiceAnnouncerFactory factory = (ServiceAnnouncerFactory) PluginManager::getSymbol("ServiceAnnouncer::create","*") ;
	 if (factory)
	   return (ServiceAnnouncer*) ((*factory)(name,type,port,textRecords,autorename,domain,host)) ;
    } catch (std::runtime_error e) {
	 std::cerr << e.what() << std::endl ;
    }
    return new dummyServiceAnnouncer(name,type,port,textRecords,autorename,domain,host) ;
#endif
  }

}
