/*
 *
 * nucleo/network/rdv/ServiceFinder.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/network/rdv/ServiceFinder.H>

#ifdef __APPLE__
#include <nucleo/network/rdv/mDNSServiceFinder.H>
#endif

#include <nucleo/core/PluginManager.H>

#include <iostream>
#include <stdexcept>

namespace nucleo {

  void
  ServiceFinder::Event::debug(std::ostream& out) const {
    out << ((event==Event::FOUND) ? "found" : "lost") ;
    out << " '" << service.name << "' (" ;
    out << service.type << ", " << service.domain ;

    if (event==FOUND) {
	 out << ", " << service.ipaddr << ":" << service.port ;
	 for (TextRecords::const_iterator pair=service.textRecords.begin();
		 pair!=service.textRecords.end();
		 ++pair) {
	   out << ", " << (*pair).first << "=" << (*pair).second ;
	 }
		 
    }

    out << ")" << std::endl ;
  }

  ServiceFinder::ServiceFinder(EventQueue *q) {
    if (!q)
	 throw std::runtime_error("ServiceFinder: queue can't be null") ;
    queue = q ;
  }

  ServiceFinder *
  ServiceFinder::create(const char *type, EventQueue *queue, const char *domain) {
#ifdef __APPLE__
    return new mDNSServiceFinder(type,queue,domain) ;
#else
    try {
	 ServiceFinderFactory factory = (ServiceFinderFactory) PluginManager::getSymbol("ServiceFinder::create","*") ;
	 if (factory)
	   return (ServiceFinder*) ((*factory)(type,queue,domain)) ;
    } catch (std::runtime_error e) {
	 std::cerr << e.what() << std::endl ;
    }
    return new ServiceFinder(queue) ;
#endif
  }

}
